/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "WheelEvent.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMWindow.h"
#include "gdom/GdomUIEvent.h"
#include "gdom/GdomWheelEvent.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDOMWindow.h"
#include "gdom/PrivateGdomUIEvent.h"
#include "gdom/PrivateGdomWheelEvent.h"
#include "gdom/PrivateIntGdomDOMWindow.h"
#include "gdom/PrivateIntGdomWheelEvent.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMWheelEvent::GDOMWheelEvent(PassRefPtr<WebCore::WheelEvent> impl)
    : GDOMUIEvent(impl)
{
}

void
gdom_wheel_event_init_wheel_event (GdomWheelEvent *thiz, glong wheel_delta_x, glong wheel_delta_y, GdomDOMWindow * view, glong screen_x, glong screen_y, glong client_x, glong client_y, gboolean ctrl_key, gboolean alt_key, gboolean shift_key, gboolean meta_key)
{
    g_return_if_fail (thiz);
    WebCore::WheelEvent * item = (gdom_wheel_event_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (view);
    WebCore::DOMWindow * _g_view = (gdom_dom_window_get_private(view));
    g_return_if_fail (_g_view);
    item->initWheelEvent(wheel_delta_x, wheel_delta_y, _g_view, screen_x, screen_y, client_x, client_y, ctrl_key, alt_key, shift_key, meta_key);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "WheelEvent.h"
#include <gdom/GdomWheelEvent.h>
#include <gdom/PrivateGdomWheelEvent.h>
#include <gdom/PrivateIntGdomWheelEvent.h>
#include <gdom/GdomWheelEventprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomWheelEvent, gdom_wheel_event, GDOM_TYPE_UI_EVENT);

/*
struct _GdomWheelEventPrivate {
    class WebCore::WheelEvent *item;
};

#define GDOM_WHEEL_EVENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_WHEEL_EVENT, GdomWheelEventPrivate))
*/
G_END_DECLS
WebCore::WheelEvent *
gdom_wheel_event_get_private(GdomWheelEvent *request)
{
    GDOMWheelEvent *res;
    res = static_cast<GDOMWheelEvent*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::WheelEvent*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_SCREEN_X,
    PROP_SCREEN_Y,
    PROP_CLIENT_X,
    PROP_CLIENT_Y,
    PROP_CTRL_KEY,
    PROP_SHIFT_KEY,
    PROP_ALT_KEY,
    PROP_META_KEY,
    PROP_WHEEL_DELTA,
    PROP_WHEEL_DELTA_X,
    PROP_WHEEL_DELTA_Y,
    PROP_OFFSET_X,
    PROP_OFFSET_Y,
    PROP_X,
    PROP_Y,
};


static void gdom_wheel_event_finalize(GObject* object)
{
    /*GdomWheelEvent* request = GDOM_WHEEL_EVENT(object);*/
    /*GdomWheelEventPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_UI_EVENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_wheel_event_parent_class)->finalize(object);
}

static void gdom_wheel_event__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomWheelEvent *self = GDOM_WHEEL_EVENT(object);
    WebCore::WheelEvent * gself = (gdom_wheel_event_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_wheel_event__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomWheelEvent *self = GDOM_WHEEL_EVENT(object);
    WebCore::WheelEvent * gself = (gdom_wheel_event_get_private(self));
    switch (prop_id) {
    case PROP_SCREEN_X:
    {
         g_value_set_long (value, gself->screenX());
         break;
    }
    case PROP_SCREEN_Y:
    {
         g_value_set_long (value, gself->screenY());
         break;
    }
    case PROP_CLIENT_X:
    {
         g_value_set_long (value, gself->clientX());
         break;
    }
    case PROP_CLIENT_Y:
    {
         g_value_set_long (value, gself->clientY());
         break;
    }
    case PROP_CTRL_KEY:
    {
         g_value_set_boolean (value, gself->ctrlKey());
         break;
    }
    case PROP_SHIFT_KEY:
    {
         g_value_set_boolean (value, gself->shiftKey());
         break;
    }
    case PROP_ALT_KEY:
    {
         g_value_set_boolean (value, gself->altKey());
         break;
    }
    case PROP_META_KEY:
    {
         g_value_set_boolean (value, gself->metaKey());
         break;
    }
    case PROP_WHEEL_DELTA:
    {
         g_value_set_long (value, gself->wheelDelta());
         break;
    }
    case PROP_WHEEL_DELTA_X:
    {
         g_value_set_long (value, gself->wheelDeltaX());
         break;
    }
    case PROP_WHEEL_DELTA_Y:
    {
         g_value_set_long (value, gself->wheelDeltaY());
         break;
    }
    case PROP_OFFSET_X:
    {
         g_value_set_long (value, gself->offsetX());
         break;
    }
    case PROP_OFFSET_Y:
    {
         g_value_set_long (value, gself->offsetY());
         break;
    }
    case PROP_X:
    {
         g_value_set_long (value, gself->x());
         break;
    }
    case PROP_Y:
    {
         g_value_set_long (value, gself->y());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_wheel_event_class_init(GdomWheelEventClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_wheel_event_finalize;
    g_klass->set_property = gdom_wheel_event__set_property_fn;
    g_klass->get_property = gdom_wheel_event__get_property_fn;

    g_object_class_install_property (g_klass, PROP_SCREEN_X,
            g_param_spec_long ("screen_x", /* name */
                "wheel_event_screen_x", /* short description */
                "read-only  glong WheelEvent.screen_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SCREEN_Y,
            g_param_spec_long ("screen_y", /* name */
                "wheel_event_screen_y", /* short description */
                "read-only  glong WheelEvent.screen_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CLIENT_X,
            g_param_spec_long ("client_x", /* name */
                "wheel_event_client_x", /* short description */
                "read-only  glong WheelEvent.client_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CLIENT_Y,
            g_param_spec_long ("client_y", /* name */
                "wheel_event_client_y", /* short description */
                "read-only  glong WheelEvent.client_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CTRL_KEY,
            g_param_spec_boolean ("ctrl_key", /* name */
                "wheel_event_ctrl_key", /* short description */
                "read-only  gboolean WheelEvent.ctrl_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SHIFT_KEY,
            g_param_spec_boolean ("shift_key", /* name */
                "wheel_event_shift_key", /* short description */
                "read-only  gboolean WheelEvent.shift_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ALT_KEY,
            g_param_spec_boolean ("alt_key", /* name */
                "wheel_event_alt_key", /* short description */
                "read-only  gboolean WheelEvent.alt_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_META_KEY,
            g_param_spec_boolean ("meta_key", /* name */
                "wheel_event_meta_key", /* short description */
                "read-only  gboolean WheelEvent.meta_key", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_WHEEL_DELTA,
            g_param_spec_long ("wheel_delta", /* name */
                "wheel_event_wheel_delta", /* short description */
                "read-only  glong WheelEvent.wheel_delta", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_WHEEL_DELTA_X,
            g_param_spec_long ("wheel_delta_x", /* name */
                "wheel_event_wheel_delta_x", /* short description */
                "read-only  glong WheelEvent.wheel_delta_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_WHEEL_DELTA_Y,
            g_param_spec_long ("wheel_delta_y", /* name */
                "wheel_event_wheel_delta_y", /* short description */
                "read-only  glong WheelEvent.wheel_delta_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSET_X,
            g_param_spec_long ("offset_x", /* name */
                "wheel_event_offset_x", /* short description */
                "read-only  glong WheelEvent.offset_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSET_Y,
            g_param_spec_long ("offset_y", /* name */
                "wheel_event_offset_y", /* short description */
                "read-only  glong WheelEvent.offset_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_X,
            g_param_spec_long ("x", /* name */
                "wheel_event_x", /* short description */
                "read-only  glong WheelEvent.x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_Y,
            g_param_spec_long ("y", /* name */
                "wheel_event_y", /* short description */
                "read-only  glong WheelEvent.y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomWheelEventPrivate));*/
}

static void gdom_wheel_event_init(GdomWheelEvent* request)
{
    /*GdomWheelEventPrivate* priv = GDOM_WHEEL_EVENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

